/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */
#include <errno.h>
#include <cortex_m/tz.h>
#include <misc/reboot.h>
#include <autoconf.h>
#include <secure_services.h>

#include <mbedtls/pk.h>

#include <mbedtls/entropy.h>
#include <mbedtls/ctr_drbg.h>
#include <mbedtls/bignum.h>
#include <mbedtls/x509.h>
#include <mbedtls/rsa.h>

/*
 * Secure Entry functions to allow access to secure services from non-secure
 * firmware.
 *
 * Note: the function will be located in a Non-Secure
 * Callable region of the Secure Firmware Image.
 */

#ifdef CONFIG_SPM_SERVICE_RNG
#ifdef MBEDTLS_CONFIG_FILE
#include MBEDTLS_CONFIG_FILE
#else
#include "mbedtls/config.h"
#endif /* MBEDTLS_CONFIG_FILE */

#include <mbedtls/platform.h>
#include <mbedtls/entropy_poll.h>

/** @brief Structure holding the memory required to generate entropy
 */
static mbedtls_rng_workbuf_internal rng_workbuf;
#endif

int printf_RTT(const char *fmt,...) {
  //char buffer[128];  va_list args;  va_start (args, fmt);  int n = vsnprintf(buffer, sizeof(buffer), fmt, args);  SEGGER_RTT_Write(0, buffer, n);  va_end(args);  return n;
  return 0;
  }



int spm_secure_services_init(void)
{
	int err = 0;

#ifdef CONFIG_SPM_SERVICE_RNG
	mbedtls_platform_context platform_ctx = {
		.p_rnd_workbuf = &rng_workbuf
	};
	err = mbedtls_platform_setup(&platform_ctx);


        SEGGER_RTT_Init();
#endif
	return err;
}




#ifdef CONFIG_SPM_SERVICE_REBOOT
__TZ_NONSECURE_ENTRY_FUNC
void spm_request_system_reboot(void)
{
	sys_reboot(SYS_REBOOT_COLD);
}
#endif /* CONFIG_SPM_SERVICE_REBOOT */

        int mbedtls_rsa_self_test( int verbose );

#ifdef CONFIG_SPM_SERVICE_RNG
__TZ_NONSECURE_ENTRY_FUNC
int spm_request_random_number(u8_t *output, size_t len, size_t *olen)
{
	int err;

        mbedtls_printf( " failed\n  ! mbedtls_ctr_drbg_seed returned %d\n", err );


	if (len != MBEDTLS_ENTROPY_MAX_GATHER) {
		return -EINVAL;
	}

	err = mbedtls_hardware_poll(&rng_workbuf, output, len, olen);

#if 1
        if(mbedtls_rsa_self_test(1) != 0)
        {
            mbedtls_printf( "RSA failed\n");
        }
#endif
	return err;
}

#if 0


        int keysize;
        int ret = 0;

        mbedtls_rsa_context rsa;
        mbedtls_ctr_drbg_context ctr_drbg;
        keysize = 2048;
        //mbedtls_snprintf( title, sizeof( title ), "RSA-%d", keysize );
        mbedtls_printf( "RSA length %d\n\r", rsa.len);
        mbedtls_rsa_init( &rsa, MBEDTLS_RSA_PKCS_V15, 0 );

static uint8_t buf_in[64] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63};
static uint8_t buf_out[64] = {0};


        int keysize;
        int ret = 0;
        
    
    
        //mbedtls_printf("Hello");
        
        mbedtls_rsa_context rsa;
        mbedtls_ctr_drbg_context ctr_drbg;
        keysize = 2048;
        //mbedtls_snprintf( title, sizeof( title ), "RSA-%d", keysize );
                mbedtls_printf( "RSA length %d\n\r", rsa.len);
        mbedtls_rsa_init( &rsa, MBEDTLS_RSA_PKCS_V15, 0 );
        

        mbedtls_printf( "RSA length %d\n\r", rsa.len);

        mbedtls_rsa_gen_key( &rsa, mbedtls_ctr_drbg_random, NULL, keysize, 65537 );
        
        if((ret = mbedtls_rsa_public( &rsa, buf_in, buf_out )) != 0)
        {
          return ret;          
        }

#endif

#endif /* CONFIG_SPM_SERVICE_RNG */



/*
__TZ_NONSECURE_ENTRY_FUNC
int RSA_Test(unsigned char * in_buf, unsigned char * out_buf)
{
    int keysize;
    int ret = 0;
    


    //mbedtls_printf("Hello");
    
    mbedtls_rsa_context rsa;
    mbedtls_ctr_drbg_context ctr_drbg;
    keysize = 2048;
    //mbedtls_snprintf( title, sizeof( title ), "RSA-%d", keysize );
    mbedtls_rsa_init( &rsa, MBEDTLS_RSA_PKCS_V15, 0 );
    
    mbedtls_rsa_gen_key( &rsa, mbedtls_ctr_drbg_random, NULL, keysize, 65537 );
    
    if((ret = mbedtls_rsa_public( &rsa, in_buf, out_buf )) != 0)
    {
      return ret;          
    }
	
    return ret;
  
}
*/
